"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _error = require("../error");

var _createExpandingBuffer = _interopRequireDefault(require("./createExpandingBuffer"));

var _invariant = _interopRequireDefault(require("./invariant"));

var _util = require("./util");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createChannel = (buffer = (0, _createExpandingBuffer.default)()) => {
  var closed = false;
  var observers = [];

  if (process.env.NODE_ENV !== 'production') {
    (0, _invariant.default)((0, _util.anyIsRingBuffer)(buffer), 'buffer must be a RingBuffer');
  }

  function checkForbiddenStates() {
    if (closed && observers.length) {
      throw (0, _error.unexpected)({
        message: 'Cannot have a closed channel with pending observers'
      });
    }

    if (observers.length && !buffer.isEmpty()) {
      throw (0, _error.unexpected)({
        message: 'Cannot have pending observers with non empty buffer'
      });
    }
  }

  function put(input) {
    if (process.env.NODE_ENV !== 'production') {
      checkForbiddenStates();
      (0, _invariant.default)(!(0, _util.anyIsUndefined)(input), 'input was undefined');
    }

    if (closed) {
      return;
    }

    if (observers.length === 0) {
      return buffer.put(input);
    }

    var observer = observers.shift();
    observer.next(input);
  }

  function next(observer) {
    if (process.env.NODE_ENV !== 'production') {
      checkForbiddenStates();
      (0, _invariant.default)((0, _util.anyIsObserver)(observer), "channel.take's observer must be an Observer");
    }

    if (closed && buffer.isEmpty()) {
      observer.complete();
    } else if (!buffer.isEmpty()) {
      observer.next(buffer.take());
    } else {
      observers.push(observer);
      return () => {
        (0, _util.arrayRemove)(observers, value => value === observer);
      };
    }
  }

  function flush(observer) {
    if (process.env.NODE_ENV !== 'production') {
      checkForbiddenStates();
      (0, _invariant.default)((0, _util.anyIsObserver)(observer), "channel.flush's observer must be a Observer");
    }

    if (closed && buffer.isEmpty()) {
      observer.complete();
      return;
    }

    observer.next(buffer.flush());
  }

  function close() {
    if (process.env.NODE_ENV !== 'production') {
      checkForbiddenStates();
    }

    if (closed) {
      return;
    }

    closed = true;
    var arr = observers;
    observers = [];

    for (var i = 0, len = arr.length; i < len; i++) {
      var observer = arr[i];
      observer.complete();
    }
  }

  return {
    close,
    flush,
    next,
    put
  };
};

var _default = createChannel;
exports.default = _default;
//# sourceMappingURL=createChannel.js.map