"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _ImmutableMap = _interopRequireDefault(require("../classes/ImmutableMap"));

var _WeakMap = _interopRequireDefault(require("../classes/WeakMap"));

var _anyIsImmutable = _interopRequireDefault(require("./anyIsImmutable"));

var _anyIsObject = _interopRequireDefault(require("./anyIsObject"));

var _arrayForEach = _interopRequireDefault(require("./arrayForEach"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var makeCacheChain = () => ({
  strongMap: new _ImmutableMap.default(),
  weakMap: new _WeakMap.default()
});

var makeCacheLink = () => ({
  cacheChain: makeCacheChain(),
  ref: {}
});

var isWeakKey = value => (0, _anyIsObject.default)(value) && !(0, _anyIsImmutable.default)(value);

var setCacheKey = (cacheChain, key, value) => {
  if (isWeakKey(key)) {
    cacheChain.weakMap.set(key, value);
  } else {
    cacheChain.strongMap = cacheChain.strongMap.set(key, value);
  }
};

var getCacheKey = (cacheChain, key) => {
  if (isWeakKey(key)) {
    return cacheChain.weakMap.get(key);
  }

  return cacheChain.strongMap.get(key);
};

var linkCacheKey = (cacheChain, key) => {
  var link = getCacheKey(cacheChain, key);

  if (!link) {
    link = makeCacheLink();
    setCacheKey(cacheChain, key, link);
  }

  return link;
};

var cache = makeCacheChain();
/**
 * This method generates a specific object instance for use in a WeakMap cache.
 * The object instance is unique based upon the parameters that are passed to
 * the this method.
 *
 * The main use of this method is for generating cache keys for memoization and
 * automatically clearing the cache when a value no longer exists in memory.
 *
 * When a non immutable object is passed as an argument it will be stored into a
 * WeakMap as part of a chain. If that object is ever removed from memory all
 * cache chains connected to the object will automatically be removed from the cache.
 *
 * @private
 * @function
 * @since v0.1.0
 * @category lang.util
 * @param {...*} args The arguments to generate a cache key for
 * @returns {Object} The cache key
 * @example
 *
 *
 */

var cacheChain = (...args) => {
  var chain = cache;
  var link = linkCacheKey(chain);
  (0, _arrayForEach.default)(args, arg => {
    link = linkCacheKey(chain, arg);
    chain = link.cacheChain;
  });
  return link.ref;
};

var _default = cacheChain;
exports.default = _default;
//# sourceMappingURL=cacheChain.js.map