"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _WeakMap = _interopRequireDefault(require("../classes/WeakMap"));

var _cacheChain = _interopRequireDefault(require("./cacheChain"));

var _functionDefineLength = _interopRequireDefault(require("./functionDefineLength"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A customisable version of [`functionMemoize`](#functionMemoize).
 * `functionMemoizeWith` takes an additional function that will be applied to a
 * given argument set and used to create the cache key under which the results
 * of the function to be memoized will be stored. Care must be taken when
 * implementing key generation to avoid clashes that may overwrite previous
 * entries erroneously.
 *
 * @private
 * @function
 * @since v0.1.0
 * @category lang.util
 * @param {Function} func The function to memoize.
 * @param {Function} cacheFunc The function to generate the cache key.
 * @return {Function} Memoized version of `func`.
 * @example
 *
 * let count = 0
 * const factorial = functionMemoizeWith(n => {
 *   count += 1
 *   return product(range(1, n + 1))
 * }, identity)
 *
 * factorial(5)
 * //=> 120
 *
 * factorial(5)
 * //=> 120
 *
 * factorial(5)
 * //=> 120
 *
 * count
 * //=> 1
 */
var functionMemoizeWith = (func, cacheFunc = _cacheChain.default) => {
  var cache = new _WeakMap.default();
  return (0, _functionDefineLength.default)(function () {
    var key = cacheFunc.apply(this, arguments);

    if (!cache.has(key)) {
      cache.set(key, func.apply(this, arguments));
    }

    return cache.get(key);
  }, func.length);
};

var _default = functionMemoizeWith;
exports.default = _default;
//# sourceMappingURL=functionMemoizeWith.js.map